function [I, DCT, rDCT, I2] = compressimage( image_name, mask_id )
% Function to compress an image and return relevant results. It takes two
% parameters:
%             image_name - the name of the image to compress
%             mask_id - this parameter can take two types of values. One is
%             numeric (1,2,3...). If it is in this form, the getmask
%             function will be called to grab one of the masks I've written
%             out already. If the mask_id variable is a mask itself (this
%             is the case in tests.m), then that mask will just be used
% It will output 4 variables:
%             I - a double array of image data from the original image
%             DCT - the resulting matrix after the DCT is applied to I
%             rDCT - the resulting matrix from zeroing out values in DCT
%             I2 - a double array of image data for the compressed image
% Some code borrowed from http://www.mathworks.com/help/images/discrete-cosine-transform.html

% If mask_id is a mask, use it. Otherwise, call getmask
if length(mask_id) > 1
    mask = mask_id;
else
    mask = getmask(mask_id);
end

% Read image data
I = imread(image_name);
I = im2double(I);

% Take the DCT 
size = length(mask);
T = dctmtx(size);
dct_function = @(block) T * block.data * T';
DCT = blockproc(I,[size size],dct_function);

% Zero out values in the DCT matrix to make a reduced DCT
rDCT = blockproc(DCT,[size size],@(block) mask .* block.data);

% Compute the inverse DCT and return the compressed image data
idct_function = @(block) T' * block.data * T;
I2 = blockproc(rDCT,[size size],idct_function);

% Print out the block size
fprintf('The block size is %dx%d\n', size, size)
